"""
  File:        simulation.py
  Author:      Petr `s3rvac` Zemek, s3rvac [at] seznam [dot] cz
  Date:        30.5.2007
  Description: Main simulation class including grid, simulation methods etc.
"""

import sys
import random
import objects
import options

class Simulation:
	# Borders
	borderSize = 2

	"""Main simulation class"""
	def __init__(self, areaSize, probCatch):
		"""Constructor

		areaSize - area size (without borders)
		probCatch - probability of a tree in a cell catching
		            fire if a tree in a neighboring cell is on fire"""
		self.originalAreaSize = areaSize
		self.originalProbCatch = probCatch
		self.createSimulationGrid(self.originalAreaSize, self.originalProbCatch)

	def createSimulationGrid(self, areaSize, probCatch):
		"""Creates simulation grid

		areaSize - area size (without borders)
		probCatch - probability of a tree in a cell catching
		            fire if a tree in a neighboring cell is on fire
		"""
		self.areaSize = areaSize + Simulation.borderSize
		# Empty grid
		self.grid = self.areaSize * [[]]
		for i in range(self.areaSize):
			self.grid[i] = self.areaSize * [objects.ObjectEmpty()]
		# Trees inside it
		for i in range(self.areaSize)[1:-1]:
			for j in range(self.areaSize)[1:-1]:
				if random.random() < options.options["probTree"]:
					burning = random.random() < options.options["probBurning"]
					self.grid[i][j] = objects.ObjectTree(probCatch, burning)

		# Burning tree in the middle
		self.grid[self.areaSize / 2][self.areaSize / 2] = objects.ObjectTree(probCatch, burning = True)

	def willBeBurning(self, tree, i, j):
		"""Returns true if the selected tree on the selected position will be burning

		tree - selected tree
		i    - row in the grid
		j    - column in the grid
		"""
		# random() produces a float between 0.0 and 1.0
		neighbourhoods = {4: [(-1, 0), (1, 0), (0, -1), (0, 1)],
			8: [(-1, -1), (-1, 0), (-1, 1), (0, 1), (1, 1), (1, 0), (1, -1), (0, -1), ]}
		# Check for catching fire from a neighbouring tree
		for x, y in neighbourhoods[options.options["neighbourhood"]]:
			if not self.grid[i + x][j + y].empty() and self.grid[i + x][j + y].isBurning():
				return random.random() < self.grid[i][j].getProbCatch()
		# Check for catching fire from a lightning
		return random.random() < (self.grid[i][j].getProbCatch() * options.options["probLightning"])

	def applySpread(self):
		"""Makes one simulation step and returns true if there was a change on the grid"""
		burnedTrees = []
		willBurnTrees = []
		for i in range(self.areaSize)[1:-1]:
			for j in range(self.areaSize)[1:-1]:
				if not self.grid[i][j].empty():
					if self.grid[i][j].isBurning():
						burnedTrees.append((i, j))
					else:
						if self.willBeBurning(self.grid[i][j], i, j):
							willBurnTrees.append((i, j))
		# Erase burned trees
		for i, j in burnedTrees:
			self.grid[i][j] = objects.ObjectEmpty()
		# Make trees (that will burn) burning
		for i, j in willBurnTrees:
			self.grid[i][j].setBurning()

		return burnedTrees != [] or willBurnTrees != []

	def restoreOriginalState(self):
		"""Restores original state of the simulation"""
		self.createSimulationGrid(self.originalAreaSize, self.originalProbCatch)

	def getRange(self):
		"""Returns inner grid range so it's possible
		to go through the grid in a loop
		"""
		return range(self.areaSize)

	def getAreaSize(self):
		"""Returns simulation area size"""
		return self.areaSize

	def getTree(self, i, j):
		"""Returns tree on the selected coordinates

		i, j - coordinates in the simulation grid"""
		return self.grid[i][j]

# Initialize the basic random number generator
random.seed()

# End of file: simulation.py
