"""
  File:        SimWin.py
  Author:      Petr `s3rvac` Zemek, s3rvac [at] seznam [dot] cz
  Date:        30.5.2007
  Description: GUI for the simulation
"""

import os
import time
import thread
import options
import common
from Tkinter import *
import tkMessageBox
from StatusBar import StatusBar

class SimWin:
	"""Main simulation window"""

	# Root TK window
	rootWin = Tk()

	def __init__(self, sim):
		"""Constructor

		sim - initialized simulation"""
		SimWin.rootWin.title(common.getProgramTitle())
		iconFile = os.path.join(options.options["iconsPath"], "fire.xbm")
		if os.path.isfile(iconFile):
			SimWin.rootWin.iconbitmap("@" + iconFile)

		# Some grid variables (defaults)
		self.winSize = 500 # Size of the window in pixels
		self.gridStep = 10 # One cell size
		self.gridBorder = 10 # Border size
		self.gridLength = 10 # Number of cells
		self.gridBg = "white" # Background color

		self.sim = sim
		self.simRunning = False # Simulation can't be running
		self.interval = options.options["interval"] * 0.001
		self.gridLength = self.sim.getAreaSize()

		self.frame = Frame(SimWin.rootWin)
		self.frame.pack()

		self.createMenu()
		self.createToolbar()
		self.createStatusbar()
		self.createSimulationGrid()

	def createMenu(self):
		"""Creates main menu"""
		menu = Menu(SimWin.rootWin)
		SimWin.rootWin.config(menu = menu)

		fileMenu = Menu(menu)
		menu.add_cascade(label = "File", menu = fileMenu)
		fileMenu.add_command(label = "Exit", command = SimWin.rootWin.destroy)

		simMenu = Menu(menu)
		menu.add_cascade(label = "Simulation", menu = simMenu)
		simMenu.add_command(label = "Apply spread", command = self.applySpread)
		simMenu.add_command(label = "Start simulation", command = self.startSimulation)
		simMenu.add_command(label = "Stop simulation", command = self.stopSimulation)
		simMenu.add_command(label = "Restart simulation", command = self.restartSimulation)

		aboutMenu = Menu(menu)
		menu.add_cascade(label = "About", menu = aboutMenu)
		aboutMenu.add_command(label = "About", command = self.showAboutMessage)

	def createToolbar(self):
		"""Creates toolbar"""
		toolbar = Frame(SimWin.rootWin)

		makeStepBut = Button(toolbar, text = "Apply spread", width = 12, command = self.applySpread)
		makeStepBut.pack(side = LEFT, padx = 2, pady = 2)

		startSimBut = Button(toolbar, text = "Start simulation", width = 12, command = self.startSimulation)
		startSimBut.pack(side = LEFT, padx = 2, pady = 2)

		stopSimBut = Button(toolbar, text = "Stop simulation", width = 12, command = self.stopSimulation)
		stopSimBut.pack(side = LEFT, padx = 2, pady = 2)

		restartSimBut = Button(toolbar, text = "Restart simulation", width = 12, command = self.restartSimulation)
		restartSimBut.pack(side = LEFT, padx = 2, pady = 2)

		toolbar.pack(side = TOP, fill = X)

	def createStatusbar(self):
		"""Creates statusbar"""
		self.statusbar = StatusBar(SimWin.rootWin)
		self.statusbar.pack(side = BOTTOM, fill = X)

	def simToGridCoord(self, index):
		"""Simulation coordinates to the window grid coordinates conversion"""
		return index + 10

	def createCell(self, coords, color):
		"""Creates a cell on the selected coordinates

		coords - selected coordinates (x, y)
		color - color fill
		"""
		x = coords[0]
		y = coords[1]

		left = x * self.gridStep + self.gridBorder
		right = left + self.gridStep
		up = y * self.gridStep + self.gridBorder
		down = up + self.gridStep

		self.canvas.create_rectangle(left, up, right, down, fill = color)

	def createSimulationGrid(self):
		"""Creates simulation grid"""
		self.gridStep = self.winSize / self.sim.getAreaSize()
		self.canvas = Canvas(SimWin.rootWin, bg = self.gridBg,
			width = self.gridLength * self.gridStep + 2 * self.gridBorder,
			height = self.gridLength * self.gridStep + 2 * self.gridBorder)
		self.canvas.pack(expand = True, fill = BOTH)
		self.actualizeSimulationGrid()

	def actualizeSimulationGrid(self):
		"""Actualizess the simulation grid.
		If the simulation is not created it will create it."""
		for x in self.sim.getRange():
			for y in self.sim.getRange():
				self.createCell((x, y), self.sim.getTree(x, y).getObjectColor())
		SimWin.rootWin.update()

	def show(self):
		"""Shows simulation window and enters main event loop"""
		SimWin.rootWin.mainloop()

	#------------------------------ Callbacks -----------------------------#

	def restartSimulation(self):
		"""Recreates the simulation"""
		self.sim.restoreOriginalState()
		self.actualizeSimulationGrid()

	def applySpread(self):
		"""Makes one simulation step and returns true if the grid was updated."""
		updated = self.sim.applySpread()
		self.actualizeSimulationGrid()
		return updated

	def startSimulation(self):
		"""Starts simulation"""
		self.running = True
		self.progressSimulation()

	def progressSimulation(self):
		"""Makes one simulation step and then waits for
		selected interval (msecs) ammout of time"""
		while self.applySpread() and self.running:
			self.after(self.interval, self.progressSimulation)

	def stopSimulation(self):
		"""Stops simulation"""
		self.running = False

	def showAboutMessage(self):
		"""Displays about dialog (message)"""
		tkMessageBox.showinfo(
			"About %s" % common.getProgramName(),
			"%s - version %s\n\n" \
			"Real-world example simulating spread of fire " \
            "across a terrain, depending on humidity etc.\n\n" \
			"Author: Petr `s3rvac` Zemek, s3rvac [at] seznam [dot] cz"
			% (common.getProgramName(), common.getVersion()))

	#---------------------------- Delay handling -----------------------------#


	def after(self, wait, func):
		"""Passed time in msecs, function to call.
		After that delay (wait) it runs the function
		in a new thread, so it won't hang main script."""
		# TODO - find better way how to do this...
		#thread.start_new_thread(self.delay, (wait, func))
		time.sleep(wait)
		func()

	def delay(self, wait, func):
		"""Waits for selected ammount of time (msec) and then it calls func"""
		time.sleep(wait)
		func()

# End of file: frame.py
