"""
  File:        options.py
  Author:      Petr `s3rvac` Zemek, s3rvac [at] seznam [dot] cz
  Date:        28.5.2007
  Description: Module containing simulation options
"""

import os
import getopt
import messages
from fexceptions import *

# Program options (default)
options = {"areaSize": 30,
	"probCatch": 0.5,
	"probLightning": 0.0,
	"probTree": 1.0,
	"probBurning": 0.0,
	"interval": 200,
	"console": False,
	"neighbourhood": 4,
	"iconsPath": os.path.normpath(os.path.join(os.getcwd(), os.path.pardir, "icons"))}

# Area sizes
_MIN_AREA_SIZE = 1
_MAX_AREA_SIZE = 100
def getMinAreaSize():
	return _MIN_AREA_SIZE
def getMaxAreaSize():
	return _MAX_AREA_SIZE

# Prob catch values
_MIN_PROBCATCH = 0.0
_MAX_PROBCATCH = 1.0
def getMinProbCatch():
	return _MIN_PROBCATCH
def getMaxProbCatch():
	return _MAX_PROBCATCH

# Prob lightning values
_MIN_PROBLIGHTNING = 0.0
_MAX_PROBLIGHTNING = 1.0
def getMinProbLightning():
	return _MIN_PROBLIGHTNING
def getMaxProbLightning():
	return _MAX_PROBLIGHTNING

# Prob tree values
_MIN_PROBTREE = 0.0
_MAX_PROBTREE = 1.0
def getMinProbTree():
	return _MIN_PROBTREE
def getMaxProbTree():
	return _MAX_PROBTREE

# Prob burning values
_MIN_PROBBURNING = 0.0
_MAX_PROBBURNING = 1.0
def getMinProbBurning():
	return _MIN_PROBBURNING
def getMaxProbBurning():
	return _MAX_PROBBURNING

# Intervals
_MIN_INTERVAL = 100
_MAX_INTERVAL = 10000
def getMinInterval():
	return _MIN_INTERVAL
def getMaxInterval():
	return _MAX_INTERVAL

# Neighbourhoods
_POSSIBLE_NEIGHBOURHOODS = (4, 8)
def getPossibleNeighbourhoods():
	return _POSSIBLE_NEIGHBOURHOODS

def _isValidAreaSize(areaSize):
	"""Validates area size"""
	return getMinAreaSize() <= areaSize <= getMaxAreaSize()

def _isValidProbCatch(probCatch):
	"""Validate prob catch value"""
	return getMinProbCatch() <= probCatch <= getMaxProbCatch()

def _isValidProbLightning(probLightning):
	"""Validate prob lightning value"""
	return getMinProbLightning() <= probLightning <= getMaxProbLightning()

def _isValidProbTree(probTree):
	"""Validate prob tree value"""
	return getMinProbTree() <= probTree <= getMaxProbTree()

def _isValidProbBurning(probBurning):
	"""Validate prob tree value"""
	return getMinProbBurning() <= probBurning <= getMaxProbBurning()

def _isValidInterval(interval):
	"""Validates interval"""
	return getMinInterval() <= interval <= getMaxInterval()

def _isValidNeighbourhood(neighbourhood):
	"""Validates neighbourhood"""
	return neighbourhood in getPossibleNeighbourhoods()

def analyzeOptions(argv):
	"""Analyzes program parameters and sets appropriate options
	argv - program parameters excluding program name (sys.argv[0])
	"""
	try:
		(opts, args) = getopt.getopt(argv, "a:c:l:t:b:i:n:o",
			["area-size=", "prob-catch=", "prob-lightning=", "prob-tree=", "prob-burning=",
			"interval=", "neighbourhood=", "console", "help", "version"])
		if args != []:
			raise OptionsExceptionOptionError("option %s not recognized" % args[0])
		for opt, val in opts:
			if opt == "--help":
				messages.printHelp()
				raise OptionsExceptionMessageShowed()
			elif opt == "--version":
				messages.printVersion()
				raise OptionsExceptionMessageShowed()
			elif opt == "-a" or opt == "--area-size":
				try:
					areaSize = int(val)
					if not _isValidAreaSize(areaSize):
						raise OptionsExceptionOptionError("%s value %s is invalid" % (opt, areaSize))
					options["areaSize"] = areaSize
				except ValueError:
					raise OptionsExceptionOptionError("%s must be an integer" % opt)
			elif opt == "-c" or opt == "--prob-catch":
				try:
					probCatch = float(val)
					if not _isValidProbCatch(probCatch):
						raise OptionsExceptionOptionError("%s value %s is invalid" % (opt, probCatch))
					options["probCatch"] = probCatch
				except ValueError:
					raise OptionsExceptionOptionError("%s must be a float" % opt)
			elif opt == "-l" or opt == "--prob-lightning":
				try:
					probLightning = float(val)
					if not _isValidProbLightning(probLightning):
						raise OptionsExceptionOptionError("%s value %s is invalid" % (opt, probLightning))
					options["probLightning"] = probLightning
				except ValueError:
					raise OptionsExceptionOptionError("%s must be a float" % opt)
			elif opt == "-t" or opt == "--prob-tree":
				try:
					probTree = float(val)
					if not _isValidProbTree(probTree):
						raise OptionsExceptionOptionError("%s value %s is invalid" % (opt, probTree))
					options["probTree"] = probTree
				except ValueError:
					raise OptionsExceptionOptionError("%s must be a float" % opt)
			elif opt == "-b" or opt == "--prob-burning":
				try:
					probBurning = float(val)
					if not _isValidProbBurning(probBurning):
						raise OptionsExceptionOptionError("%s value %s is invalid" % (opt, probBurning))
					options["probBurning"] = probBurning
				except ValueError:
					raise OptionsExceptionOptionError("%s must be a float" % opt)
			elif opt == "-i" or opt == "--interval":
				try:
					interval = int(val)
					if not _isValidInterval(interval):
						raise OptionsExceptionOptionError("%s value %s is invalid" % (opt, interval))
					options["interval"] = interval
				except ValueError:
					raise OptionsExceptionOptionError("%s must be an integer" % opt)
			elif opt == "-n" or opt == "--neighbourhood":
				try:
					neighbourhood = int(val)
					if not _isValidNeighbourhood(neighbourhood):
						raise OptionsExceptionOptionError("%s value %s is invalid" % (opt, neighbourhood))
					options["neighbourhood"] = neighbourhood
				except ValueError:
					raise OptionsExceptionOptionError("%s must be an integer" % opt)
			elif opt == "-o" or opt == "--console":
				options["console"] = True
	except getopt.GetoptError, e:
		raise OptionsExceptionOptionError(e.msg)

# End of file: options.py
